' See Eric W. Weisstein's article "Newton's Method."
' From MathWorld--A Wolfram Web Resource. 
' http://mathworld.wolfram.com/NewtonsMethod.html 

Imports System.Math
Imports System.Drawing.Drawing2D

Public Class Form1
    Inherits System.Windows.Forms.Form

#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'Form overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        '
        'Form1
        '
        Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
        Me.ClientSize = New System.Drawing.Size(456, 413)
        Me.Name = "Form1"
        Me.Text = "Form1"

    End Sub

#End Region

    Private Const SCALE_FACTOR As Single = 20

    ' The function.
    Private Function F(ByVal x As Double) As Single
        Return x * x * x / 3 - 2 * x * x + 5
    End Function

    ' The function's derivative.
    Private Function dFdx(ByVal x As Double) As Single
        Return x * x - 4 * x
    End Function

    Private Sub Form1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MyBase.Paint
        DrawGraph(e.Graphics)
    End Sub

    ' Draw the background graph.
    Private Sub DrawGraph(ByVal gr As Graphics)
        gr.Clear(Me.BackColor)

        ' Scale for convenience.
        gr.ScaleTransform(SCALE_FACTOR, -SCALE_FACTOR)
        gr.TranslateTransform( _
            Me.ClientRectangle.Width / 2, _
            Me.ClientRectangle.Height / 2, _
            Drawing2D.MatrixOrder.Append)

        ' Draw the axes.
        Dim blue_pen As New Pen(Color.Blue, 0)
        For x As Integer = -10 To 10
            gr.DrawLine(blue_pen, x, -0.5!, x, 0.5!)
        Next x
        For y As Integer = -10 To 10
            gr.DrawLine(blue_pen, -0.5!, y, 0.5!, y)
        Next y
        gr.DrawLine(blue_pen, -10, 0, 10, 0)
        gr.DrawLine(blue_pen, 0, -10, 0, 10)
        blue_pen.Dispose()

        ' Draw the curve.
        Dim red_pen As New Pen(Color.Red, 0)
        Dim y1 As Single
        Dim y2 As Single = F(-10)
        Const STEP_SIZE As Single = 0.1
        For x As Single = -10 + STEP_SIZE To 10 Step STEP_SIZE
            y1 = y2
            y2 = F(x)
            gr.DrawLine(red_pen, x - STEP_SIZE, y1, x, y2)
        Next x
        red_pen.Dispose()
    End Sub

    ' Find the roots by using Newton's method.
    Private Sub Form1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        ' Clear previous results.
        Dim gr As Graphics = Me.CreateGraphics()
        DrawGraph(gr)

        ' Convert the X coordinate into world coordinates.
        Dim m As Matrix = gr.Transform()
        m.Invert()
        Dim pts() As PointF = {New PointF(e.X, e.Y)}
        m.TransformPoints(pts)
        Dim x0 As Single = pts(0).X

        Const CUTOFF As Single = 0.0000001
        Dim epsilon As Single
        Dim the_pen As New Pen(Color.Black, 0)
        Dim iterations As Integer = 0
        Do
            iterations += 1
            gr.DrawLine(the_pen, x0, -1.5!, x0, 1.5!)
            epsilon = -F(x0) / dFdx(x0)
            x0 = x0 + epsilon
        Loop While Abs(epsilon) > CUTOFF
        gr.DrawLine(the_pen, x0, -3.0!, x0, 3.0!)
        the_pen.Dispose()

        Me.Text = x0 & " +/-" & epsilon & " in " & iterations & " iterations"
    End Sub
End Class
